package ch.zhaw.gartenverwaltung.plantList;

import ch.zhaw.gartenverwaltung.io.HardinessZoneNotSetException;
import ch.zhaw.gartenverwaltung.io.JsonPlantDatabase;
import ch.zhaw.gartenverwaltung.io.PlantDatabase;
import ch.zhaw.gartenverwaltung.types.HardinessZone;
import ch.zhaw.gartenverwaltung.types.Plant;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.function.Predicate;

public class PlantListModel {
    private final PlantDatabase plantDatabase;
    private HardinessZone currentZone;

    /**
     * Comparators to create sorted Plant List
     */
    public final Comparator<Plant> sortByName = (Plant o1, Plant o2) -> o1.name().compareTo(o2.name());
    public final Comparator<Plant> getSortById = (Plant o1, Plant o2) -> Long.compare(o1.id(), o2.id());

    public PlantListModel() {
        plantDatabase = new JsonPlantDatabase();
    }

    public void setCurrentZone(HardinessZone currentZone) {
        this.currentZone = currentZone;
    }

    public HardinessZone getCurrentZone() {
        return currentZone;
    }

    /**
     * Method to get actual Plant List in alphabetic Order
     * @return actual Plant List in alphabetic Order
     */
    public List<Plant> getPlantList(HardinessZone zone) throws HardinessZoneNotSetException, IOException {
        setCurrentZone(zone);
        return getSortedPlantList(zone, sortByName);
    }


    /**
     * Method to get the actual Plant list in custom Order
     * @param zone selected hardiness zone
     * @param comparator comparator to sort the list
     * @return sorted list with plants in the given hardiness zone
     * @throws IOException If the database cannot be accessed
     * @throws HardinessZoneNotSetException If no {@link HardinessZone} was specified
     */
    public List<Plant> getSortedPlantList(HardinessZone zone, Comparator<Plant> comparator) throws HardinessZoneNotSetException, IOException {
        setCurrentZone(zone);
        return plantDatabase.getPlantList(zone).stream().sorted(comparator).toList();
    }

    /**
     * Method to get Filtered plant list
     * @param predicate predicate to filter the list
     * @param zone selected hardiness zone
     * @return filterd list with plants in the hardinness zone
     * @throws IOException If the database cannot be accessed
     * @throws HardinessZoneNotSetException If no {@link HardinessZone} was specified
     */
    public List<Plant> getFilteredPlantList(HardinessZone zone, Predicate<Plant> predicate) throws HardinessZoneNotSetException, IOException {
        setCurrentZone(zone);
        return getPlantList(zone).stream().filter(predicate).toList();
    }

    /**
     * Method to get Filtered plant list by id by exact match
     * @param zone selected hardiness zone
     * @param id id of plant
     * @return if id doesn't exist: empty List, else list with 1 plant entry.
     * @throws IOException If the database cannot be accessed
     * @throws HardinessZoneNotSetException If no {@link HardinessZone} was specified
     */
    public List<Plant> getFilteredPlantListById(HardinessZone zone, Long id) throws HardinessZoneNotSetException, IOException {
        setCurrentZone(zone);
        List<Plant> plantList = new ArrayList<>();
        plantDatabase.getPlantById(zone, id).ifPresent(plantList::add);
        return plantList;
    }
}
